<?
/*
|----------------------------------------------------------------------------------------------------------------------------------
| Cosine law scale generator
|----------------------------------------------------------------------------------------------------------------------------------
|
|	Generates a scale based on the Cosine Law. 
|	EG: for measuring the distance between the tips of a pair of scissors given
|	the length of the blades, and the angle between them.
|
|
*/



$g=new cosine_scale();

$g->generate();


class cosine_scale{

	var $phi=1.61803;

	var $long_arm_length;
	var $band_inner_radius;
	var $band_width=10;  
	
	
	var $page_margin=10;
	var $image_size=300;	// 300 production
	var $line_thickness=.2;

	var $font = '/usr/share/fonts/truetype/msttcorefonts/verdana.ttf';
	

	var $font_size;							// is image size / 100


	var $img;									
	var $startx;
	var $starty;
	var $mulitplier;

	var $svgStr='<svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink">';

	var $svgCSS='font-family="Verdana" fill="black"';


	var $units='inches';
//	var $units='mm';




/*
|----------------------------------------------------------------------------------------------------------------------------------
| drawScale
|----------------------------------------------------------------------------------------------------------------------------------
|
|
*/

	function drawScale($r,$direction){
	
		$done=array();


		for ($angle=0;$angle<=180; $angle=$angle+0.001) {										// go round the circle .001 of a degree at at time.


			$rads = deg2rad($angle);

			$inch_mxer=$this->units=='mm'?1:10;

			$numeric_length=number_format(sqrt(2*$r*$r-2*$r*$r*cos($rads))*$inch_mxer,2);		 // numeric length = number to print

			list($int,$zeros)=explode('.',$numeric_length);
			
			$division=$int%10;


			if (@in_array($zeros,array('00')) and !@in_array($int.'_'.$division,$done)) {			// if we've got a whole number, draw a line.

				$numeric_length=$numeric_length/$inch_mxer;

				switch ($division){
				
					case 0:

						if ($this->units=='mm') {
							$mx=$direction=='outward'?0:.26;
						}else{
							$mx=$direction=='outward'?.10:0;
						}
						

						$this->drawline($angle,$rads,$numeric_length,'with text',$this->band_width*$mx,$direction);

					break;

					case 5:

						$shrinkage=$this->units=='mm'?.5:.2;
						$this->drawline($angle,$rads,$numeric_length,'',$this->band_width*$shrinkage,$direction); 					// mid line

					break;

					default:
			
						if ($this->units!='mm') {	
							$shrinkage=$this->units=='mm'?.8:.6;
							$this->drawline($angle,$rads,$numeric_length,'',$this->band_width*$shrinkage,$direction);						// small tick	
						}

					break;

				}

				$done[]=$int.'_'.$division;
			}
		}
	
		$numeric_length=$numeric_length/$inch_mxer;
		
		if ($numeric_length!=200 and $numeric_length!=8) $this->drawline($angle,$rads,$numeric_length,'with text','',$direction,'last');
		
	}




/*
|----------------------------------------------------------------------------------------------------------------------------------
| generate
|----------------------------------------------------------------------------------------------------------------------------------
|
|
*/

	function generate(){

		
		$this->startx=$this->image_size/2+$this->page_margin;	
		$this->starty=$this->startx;
	
		$this->font_size=$this->image_size/40;

		if ($this->units=='mm') {

			$this->long_arm_length=100;
			$this->multiplier=$this->image_size/($this->long_arm_length*2);
		}else{

			$this->long_arm_length=3.93701;	
			$this->multiplier=$this->image_size/($this->long_arm_length*50);
		}
	
	
		$this->band_inner_radius=80;
		
		$this->svgStr.='   <circle cx="'.($this->startx-$this->page_margin*2).'" cy="'.$this->starty.'" r="'.($this->band_inner_radius*$this->multiplier).'"  stroke="black" fill="white" stroke-width="'.$this->line_thickness.'"  />';

		$this->drawScale($this->long_arm_length,'outward');
		$this->drawScale($this->long_arm_length/$this->phi,'inward');


		$this->svgStr.='  </svg>';
		header('Content-Type: image/svg+xml');
		print $this->svgStr;


	}



	/*
	|----------------------------------------------------------------------------------------------------------------------------------
	| drawline
	|----------------------------------------------------------------------------------------------------------------------------------
	|
	|
	*/
	function drawline($angle,$rads,$numeric_length,$with_text='',$truncate=0,$direction,$last=''){


			if ($direction=='outward') {

				$sx = ($this->startx+cos($rads)*($this->band_inner_radius)*$this->multiplier);
				$sy= ($this->starty-sin($rads)*($this->band_inner_radius)*$this->multiplier);
				
				$endx = ($this->startx+cos($rads)*($this->band_inner_radius+$this->band_width-$truncate)*$this->multiplier);
				$endy = ($this->starty-sin($rads)*($this->band_inner_radius+$this->band_width-$truncate)*$this->multiplier);
			
			}else{

				$sx = ($this->startx+cos($rads)*($this->band_inner_radius)*$this->multiplier);
				$sy= ($this->starty-sin($rads)*($this->band_inner_radius)*$this->multiplier);
				
				$endx = ($this->startx+cos($rads)*($this->band_inner_radius-$this->band_width+$truncate)*$this->multiplier);
				$endy = ($this->starty-sin($rads)*($this->band_inner_radius-$this->band_width+$truncate)*$this->multiplier);
			}


			$sx=$this->image_size-$sx;						// reverse the direction of curve
			$endx=$this->image_size-$endx;
			
			$this->svgStr.='<line x1="'.$sx.'" y1="'.$sy.'" x2="'.$endx.'" y2="'.$endy.'"  stroke-width="'.$this->line_thickness.'" style="stroke: #000000;"/>';
	

			if ($with_text) {
			
				$angle_offset=-0.01;
				$angle=$angle+180;


				if ($direction=='outward') {


				$chars=strlen(number_format($numeric_length,0))+1;
					
					$radial_offset=strlen(number_format($numeric_length,0))<3?-.1:-3.3;
					$angle_offset=strlen(number_format($numeric_length,0))<3?0.020:-0.01;

					$fontX = ($this->startx+cos($rads+$angle_offset)*($this->band_inner_radius+$this->band_width+$radial_offset)*$this->multiplier);
					$fontY = ($this->starty-sin($rads+$angle_offset)*($this->band_inner_radius+$this->band_width+$radial_offset)*$this->multiplier);
					
				}else{

					$radial_offset=strlen(number_format($numeric_length,0))<3?-4.2:-3.3;
					$angle_offset=strlen(number_format($numeric_length,0))<3?0.027:-0.01;


					$fontX = ($this->startx+cos($rads+$angle_offset)*($this->band_inner_radius-$this->band_width+$radial_offset)*$this->multiplier);
					$fontY = ($this->starty-sin($rads+$angle_offset)*($this->band_inner_radius-$this->band_width+$radial_offset)*$this->multiplier);
				
				}	

				$fontX=$this->image_size-$fontX;	// reverse the direction of curve
				$numeric_text=number_format($numeric_length,0);

			
				if ($numeric_text==0) $fontY=$fontY-4;	// put the 0 on the bar


				$this->svgStr.='<text x="'.$fontX.'" y="'.$fontY.'" '.$this->svgCSS.' font-size="'.$this->font_size.'" transform="rotate('.$angle.' '.$fontX .','.$fontY.')">'.$numeric_text.'</text>';	// angle x,y

			}
	}

}




function p($arr){
	
	print "<pre>";
	if (is_array($arr)) {
		print_r($arr);
	}else{
		print htmlentities($arr);
	}
	print "</pre>";

}

?>
